package	jp.co.sds_corp.syslog.layout;

import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.syslog.SyslogMsg;

/**
* 日付フィールドを編集するコンバータです。
*/
public class SyslogConverter implements IConverter
{
	private static final long serialVersionUID = 1L;
	enum Type {
		/** タイプ：現在日時 */
		NOW,
		/** タイプ：受信日時 */
		TIME,
		/** タイプ：IPアドレス */
		ADDRESS,
		/** タイプ：ホスト名 */
		HOSTNAME,
		/** タイプ：facility */
		FACILITY,
		/** タイプ：facility(文字列) */
		FACILITY_MSG,
		/** タイプ：priority */
		PRIORITY,
		/** タイプ：priority(文字列) */
		PRIORITY_MSG,
		/** タイプ：メッセージ */
		MESSAGE,
		/** タイプ：Originalメッセージ */
		ORG_MESSAGE
	};

	Type	type;
	String pattern;

	/**
	* 指定されたタイプとフォーマット文字列でSyslogConverterを作成します。
	* @param	type	編集タイプ
	* @param	pattern	フォーマット文字列
	*/
	public SyslogConverter( Type type, String pattern ) {
		this.type = type;
		this.pattern = pattern;
	}

	/**
	* 文字列を編集して出力します。
	* @param	msg		編集元のデータ
	* @return	編集結果の文字列
	*/
	public String convert( SyslogMsg msg ) {
		switch ( type ) {
		/** タイプ：現在日時 */
		case NOW:
			if ( pattern == null ) {
				pattern = "yyyy-MM-dd HH:mm:ss";
			}
			return new CmnDate().format( pattern );
		/** タイプ：受信日時 */
		case TIME:
			if ( pattern == null ) {
				pattern = "yyyy-MM-dd HH:mm:ss";
			}
			return msg.getDate().format( pattern );
		/** タイプ：IPアドレス */
		case ADDRESS:
			return msg.getAddress().getHostAddress();
		/** タイプ：ホスト名 */
		case HOSTNAME:
			return msg.getAddress().getHostName();
		/** タイプ：facility */
		case FACILITY:
			return String.valueOf( msg.getFacility().getValue() );
		/** タイプ：facility */
		case FACILITY_MSG:
			return msg.getFacility().toString();
		/** タイプ：priority */
		case PRIORITY:
			return String.valueOf( msg.getPriority().getValue() );
		/** タイプ：priority */
		case PRIORITY_MSG:
			return msg.getPriority().toString();
		/** タイプ：メッセージ */
		case MESSAGE:
			return msg.getMessage();
		/** タイプ：Originalメッセージ */
		case ORG_MESSAGE:
			return msg.getOriginalMessage();
		default:
			return "%" + pattern;
		}
	}
}
