package	jp.co.sds_corp.syslog.viewer;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import java.util.regex.Pattern;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;

import jp.co.sds_corp.syslog.SyslogEvent;
import jp.co.sds_corp.util.common.NoSuchCodeException;
import jp.co.sds_corp.util.syslog.Facility;
import jp.co.sds_corp.util.syslog.Priority;
import jp.co.sds_corp.util.syslog.SyslogMsg;
import jp.co.sds_corp.util.window.ValuePanel;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* syslog検索ダイアログです。
*/
public class SearchDialog extends JDialog
{
	private static final long serialVersionUID = 1L;
	static final String SEL_NONE = "(None)";
	JButton BTN_PREV;
	JButton BTN_NEXT;
	JButton BTN_CANCEL = new JButton( "Cancel" );
	ValuePanel	VAL_FAC;
	ValuePanel	VAL_PRI;
	ValuePanel	VAL_PTN;
	JCheckBox	CHK_CASE;
	JCheckBox	CHK_REGEXP;
	SyslogParam		param;
	ResourceBundle res;

	public SearchDialog( JDialog parent, SyslogParam param ) {
		super( parent );
		this.param = param;
		createControls();
	}

	public SearchDialog( JFrame parent, SyslogParam param ) {
		super( parent );
		this.param = param;
		createControls();
	}

	public SearchDialog( SyslogParam param ) {
		super();
		this.param = param;
		createControls();
	}

	protected void createControls() {
		setModal( true );
		WindowUtility.setHideOnEscape( this );
		res = ResourceBundle.getBundle( "resource.viewer" );
		String text;
		JComboBox<String>	combo;

		setResizable( false );
		text = res.getString( "search.title" );
		setTitle( text );
		setSize( 477, 200 );
		getContentPane().setLayout(
			new BoxLayout( getContentPane(), BoxLayout.Y_AXIS ) );
		setFont( param.getDefaultFont() );

		WindowUtility.setHideOnEscape( this );

		text = res.getString( "search.lbl1.text" );
		combo = new JComboBox<String>( Facility.listMessages() );
		combo.insertItemAt( SEL_NONE, 0 );
		VAL_FAC = new ValuePanel( ValuePanel.ReturnType.STRING, text, combo );
		VAL_FAC.setValue( SEL_NONE );
		getContentPane().add( VAL_FAC );
		text = res.getString( "search.lbl2.text" );
		combo = new JComboBox<String>( Priority.listMessages() );
		combo.insertItemAt( SEL_NONE, 0 );
		VAL_PRI = new ValuePanel( ValuePanel.ReturnType.STRING, text, combo );
		VAL_PRI.setValue( SEL_NONE );
		getContentPane().add( VAL_PRI );

		text = res.getString( "search.lbl3.text" );
		VAL_PTN = new ValuePanel( ValuePanel.ReturnType.STRING, text );
		getContentPane().add( VAL_PTN );

		text = res.getString( "search.lbl4.text" );
		CHK_CASE = new JCheckBox( text );
		getContentPane().add( CHK_CASE );
		text = res.getString( "search.lbl5.text" );
		CHK_REGEXP = new JCheckBox( text );
		getContentPane().add( CHK_REGEXP );

		Box box1 = new Box( BoxLayout.X_AXIS );
		text = res.getString( "search.prev.text" );
		BTN_PREV = new JButton( text );
		text = res.getString( "search.next.text" );
		BTN_NEXT = new JButton( text );
		box1.add( BTN_PREV );
		box1.add( BTN_NEXT );
		box1.add(Box.createHorizontalStrut(30));
		BTN_CANCEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onCancel();
			}
		});
		box1.add( BTN_CANCEL );
		box1.setPreferredSize( new Dimension( 450, 50 ));
		getContentPane().add( box1 );
	}

	/**
	* syslogのがこのフィルタの条件に合致するかを検査します。
	* @param    e       データ受信イベント
	*/
	public boolean match( SyslogEvent e ) {
		SyslogMsg msg = e.getMessage();
		String val;
		val = (String )VAL_FAC.getValue();
		if ( !val.equals( SEL_NONE )) {
			try {
				if ( !msg.getFacility().equals( Facility.parse( val ))) {
					return false;
				}
			} catch ( NoSuchCodeException ex ) {
			}
		}
		val = (String )VAL_PRI.getValue();
		if ( !val.equals( SEL_NONE )) {
			try {
				if ( !msg.getPriority().equals( Priority.parse( val ))) {
					return false;
				}
			} catch ( NoSuchCodeException ex ) {
			}
		}

		String pattern = (String)VAL_PTN.getValue();
		if ( pattern != null ) {
			if ( CHK_REGEXP.isSelected() ) {
				int flg = Pattern.DOTALL;
				if ( CHK_CASE.isSelected() ) {
					flg |=  Pattern.CASE_INSENSITIVE;
				}
				Pattern ptn = Pattern.compile( pattern, flg );
				if ( !ptn.matcher( msg.getMessage() ).matches() ) {
					return false;
				}
			} else {
				String s1, s2;
				if ( CHK_CASE.isSelected() ) {
					s1 = msg.getMessage().toUpperCase();
					s2 = pattern.toUpperCase();
				} else {
					s1 = msg.getMessage();
					s2 = pattern;
				}
				if ( s1.indexOf( s2 ) < 0 ) {
					return false;
				}
			}
		}
		return true;
	}

	public void onCancel() {
		setVisible( false );
	}
}
