package	jp.co.sds_corp.syslog.viewer.filter;

import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.border.TitledBorder;

import jp.co.sds_corp.syslog.filter.IFilter;
import jp.co.sds_corp.syslog.filter.LevelFilter;
import jp.co.sds_corp.util.common.NoSuchCodeException;
import jp.co.sds_corp.util.syslog.Facility;
import jp.co.sds_corp.util.syslog.Priority;
import jp.co.sds_corp.util.window.ValuePanel;

/**
* {@link LevelFilter}の設定を行います。
*/
public class LevelPanel extends AbstractPanel
{
	private static final long serialVersionUID = 1L;
	static final String SEL_NONE = "(None)";
	ValuePanel	VAL_FAC;
	ValuePanel	VAL_MAX;
	ValuePanel	VAL_MIN;
	JCheckBox	CHK_EXC;
	JCheckBox	CHK_VALID;
	ResourceBundle res;

	public LevelPanel() {
		res = ResourceBundle.getBundle( "resource.viewer" );
		String text;
		text = res.getString( "filter.level.title" );
		setBorder( new TitledBorder( text ));
		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ));

		createControls();
	}

	protected void createControls() {
		String text;
		text = res.getString( "filter.exclude.text" );
		CHK_EXC = new JCheckBox( text );
		text = res.getString( "filter.valid.text" );
		CHK_VALID = new JCheckBox( text );

		JComboBox<String> cmb;
		cmb = new JComboBox<String>( Facility.listMessages() );
		cmb.insertItemAt( SEL_NONE, 0 );
		text = res.getString( "filter.level.lbl1.text" );
		VAL_FAC = new ValuePanel( ValuePanel.ReturnType.STRING, text, cmb );
		add( VAL_FAC );
		cmb = new JComboBox<String>( Priority.listMessages() );
		cmb.insertItemAt( SEL_NONE, 0 );
		text = res.getString( "filter.level.lbl2.text" );
		VAL_MAX = new ValuePanel( ValuePanel.ReturnType.STRING, text, cmb );
		cmb = new JComboBox<String>( Priority.listMessages() );
		cmb.insertItemAt( SEL_NONE, 0 );
		add( VAL_MAX );
		text = res.getString( "filter.level.lbl3.text" );
		VAL_MIN = new ValuePanel( ValuePanel.ReturnType.STRING, text, cmb );
		add( VAL_MIN );
		add( Box.createVerticalStrut( 60 ) );

		Box box2 = new Box( BoxLayout.X_AXIS );
		box2.add( CHK_EXC );
		box2.add( Box.createHorizontalStrut( 500 ));
		add( box2 );
		Box box3 = new Box( BoxLayout.X_AXIS );
		box3.add( CHK_VALID );
		box3.add( Box.createHorizontalStrut( 500 ));
		add( box3 );
	}

	public void clear() {
		this.filter = null;
		VAL_FAC.setValue( SEL_NONE );
		VAL_MAX.setValue( SEL_NONE );
		VAL_MIN.setValue( SEL_NONE );
		CHK_EXC.setSelected( false );
		CHK_VALID.setSelected( false );
	}

	public void setFilter( IFilter filter ) {
		super.setFilter( filter );
		if ( filter == null ) {
			clear();
			return;
		}
		LevelFilter sf = (LevelFilter )filter;
		if ( sf.getFacility() == null ) {
			VAL_FAC.setValue( SEL_NONE );
		} else {
			VAL_FAC.setValue( sf.getFacility().toString() );
		}
		if ( sf.getMaxPriority() == null ) {
			VAL_MAX.setValue( SEL_NONE );
		} else {
			VAL_MAX.setValue( sf.getMaxPriority().toString() );
		}
		if ( sf.getMinPriority() == null ) {
			VAL_MIN.setValue( SEL_NONE );
		} else {
			VAL_MIN.setValue( sf.getMinPriority().toString() );
		}
		if ( sf.isExclude()) {
			CHK_EXC.setSelected( true );
		} else {
			CHK_EXC.setSelected( false );
		}
		if ( sf.isValid()) {
			CHK_VALID.setSelected( false );
		} else {
			CHK_VALID.setSelected( true );
		}
	}

	public void updateFilter() {
		LevelFilter sf;
		if ( filter == null ) {
			sf = new LevelFilter();
			filter = sf;
		} else {
			sf = (LevelFilter )filter;
		}
		String val;
		val = (String )VAL_FAC.getValue();
		if ( val.equals( SEL_NONE )) {
			sf.setFacility( null );
		} else {
			try {
				sf.setFacility( Facility.parse( val ));
			} catch ( NoSuchCodeException ex ) {
			}
		}
		val = (String )VAL_MAX.getValue();
		if ( val.equals( SEL_NONE )) {
			sf.setMaxPriority( null );
		} else {
			try {
				sf.setMaxPriority( Priority.parse( val ));
			} catch ( NoSuchCodeException ex ) {
			}
		}
		val = (String )VAL_MIN.getValue();
		if ( val.equals( SEL_NONE )) {
			sf.setMinPriority( null );
		} else {
			try {
				sf.setMinPriority( Priority.parse( val ));
			} catch ( NoSuchCodeException ex ) {
			}
		}
		sf.setExclude( CHK_EXC.isSelected());
		sf.setValid( !CHK_VALID.isSelected());
	}

}
