package	jp.co.sds_corp.syslog.viewer.filter;

import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JOptionPane;
import javax.swing.border.TitledBorder;

import jp.co.sds_corp.syslog.filter.IFilter;
import jp.co.sds_corp.syslog.filter.SourceFilter;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.window.ValuePanel;

/**
* {@link SourceFilter}の設定を行います。
*/
public class SourcePanel extends AbstractPanel
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	ValuePanel	VAL_SRC;
	JCheckBox	CHK_EXC;
	JCheckBox	CHK_VALID;
	ResourceBundle res;

	public SourcePanel() {
		res = ResourceBundle.getBundle( "resource.viewer" );
		String text;
		text = res.getString( "filter.source.title" );
		setBorder( new TitledBorder( text ));
		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ));

		createControls();
	}

	protected void createControls() {
		String text;
		text = res.getString( "filter.exclude.text" );
		CHK_EXC = new JCheckBox( text );
		text = res.getString( "filter.valid.text" );
		CHK_VALID = new JCheckBox( text );

		text = res.getString( "filter.source.lbl1.text" );
		VAL_SRC = new ValuePanel( ValuePanel.ReturnType.STRING, text );
		add( VAL_SRC );
		Box box2 = new Box( BoxLayout.X_AXIS );
		box2.add( CHK_EXC );
		box2.add( Box.createHorizontalStrut( 500 ));
		add( box2 );
		Box box3 = new Box( BoxLayout.X_AXIS );
		box3.add( CHK_VALID );
		box3.add( Box.createHorizontalStrut( 500 ));
		add( box3 );
	}

	public void clear() {
		this.filter = null;
		VAL_SRC.setValue( null );
		CHK_EXC.setSelected( false );
		CHK_VALID.setSelected( false );
	}

	public void setFilter( IFilter filter ) {
		super.setFilter( filter );
		if ( filter == null ) {
			clear();
			return;
		}
		SourceFilter sf = (SourceFilter )filter;
		VAL_SRC.setValue( sf.getSource() );
		if ( sf.isExclude()) {
			CHK_EXC.setSelected( true );
		} else {
			CHK_EXC.setSelected( false );
		}
		if ( sf.isValid()) {
			CHK_VALID.setSelected( false );
		} else {
			CHK_VALID.setSelected( true );
		}
	}

	public void updateFilter() {
		SourceFilter sf;
		if ( filter == null ) {
			sf = new SourceFilter();
			filter = sf;
		} else {
			sf = (SourceFilter )filter;
		}
		String var = (String )VAL_SRC.getValue();
		try {
			sf.setSource( var );
		} catch ( Exception ex ) {
			logger.error( this, "update error", ex );
			String msg, title;
			title = res.getString( "filter.source.err1.title" );
			msg = res.getString( "filter.source.err1.message" );
			JOptionPane.showMessageDialog( this,
				msg + "\n" + ex.getMessage(), title,
				JOptionPane.ERROR_MESSAGE );
		}
		sf.setExclude( CHK_EXC.isSelected());
		sf.setValid( !CHK_VALID.isSelected());
	}

}
