package	jp.co.sds_corp.syslog.viewer.statistics; 

import java.awt.Color;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Window;
import java.text.DecimalFormat;
import java.util.Iterator;
import java.util.ResourceBundle;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import jp.co.sds_corp.syslog.SyslogEvent;
import jp.co.sds_corp.syslog.viewer.SyslogParam;
import jp.co.sds_corp.syslog.viewer.SyslogTable;
import jp.co.sds_corp.util.syslog.Facility;
import jp.co.sds_corp.util.syslog.Priority;
import jp.co.sds_corp.util.syslog.SyslogMsg;

/**
* 全般的状況の表示パネルです。
*/
public class GeneralPanel extends JPanel
{
	private static final long serialVersionUID = 1L;
	Window		owner;
	SyslogParam		param;
	Vector<SyslogEvent>	vecLog;
	SyslogTable	table;
	ResourceBundle res;
	JButton BTN_UPD = new JButton( "Update" );
	JLabel		LBL_ALL;
	JTextField	TXT_ALL = new JTextField();
	JLabel		LBL_DISP;
	JTextField	TXT_DISP = new JTextField();
	FacilityModel model1 = new FacilityModel();
	JTable		TBL_FAC = new JTable( model1 );
	PriorityModel model2 = new PriorityModel();
	JTable		TBL_PRI = new JTable( model2 );
	TreeMap<Facility,Integer>	mapFac = new TreeMap<Facility,Integer>();
	TreeMap<Priority,Integer>	mapPri = new TreeMap<Priority,Integer>();
	ColorDefine	coldef = new ColorDefine();

	static class FacilityModel extends DefaultTableModel
	{
		private static final long serialVersionUID = 1L;
		DecimalFormat df = new DecimalFormat( "0.0 %" );
		public void add( Facility fac, Integer num, Double perc ) {
			Object[] row = { "", fac, num, perc };
			addRow( row );
		}
		public Object getValueAt( int row, int column ) {
			String ret = null;
			Vector vec = (Vector )getDataVector().get( row );
			switch ( column ) {
			case 0: ret = ""; break;
			case 1:
				Facility fac = (Facility )vec.get( 1 );
				ret = fac.toString();
				break;
			case 2:
				Integer num = (Integer )vec.get( 2 );
				ret = num.toString();
				break;
			case 3:
				Double perc = (Double )vec.get( 3 );
				ret = df.format( perc );
				break;
			}
			return ret;
		}
		public boolean isCellEditable( int row, int clm ) {
			return false;
		}
	}

	static class PriorityModel extends DefaultTableModel
	{
		private static final long serialVersionUID = 1L;
		DecimalFormat df = new DecimalFormat( "0.0 %" );

		public void add( Priority pri, Integer num, Double perc ) {
			Object[] row = { "", pri, num, perc };
			addRow( row );
		}
		public Object getValueAt( int row, int column ) {
			String ret = null;
			Vector vec = (Vector )getDataVector().get( row );
			switch ( column ) {
			case 0: ret = ""; break;
			case 1:
				Priority pri = (Priority )vec.get( 1 );
				ret = pri.toString();
				break;
			case 2:
				Integer num = (Integer )vec.get( 2 );
				ret = num.toString();
				break;
			case 3:
				Double perc = (Double )vec.get( 3 );
				ret = df.format( perc );
				break;
			}
			return ret;
		}
		public boolean isCellEditable( int row, int clm ) {
			return false;
		}
	}

	static class NumberRenderer extends DefaultTableCellRenderer
	{
		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent( JTable table,
				Object value, boolean isSelected, boolean hasFocus,
				int row, int column ) 
		{
			setText( (String )value );
			setHorizontalAlignment( JLabel.RIGHT );
			return this;
		}
	}

	class FacilityColorRenderer extends DefaultTableCellRenderer
	{
		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent( JTable table,
				Object value, boolean isSelected, boolean hasFocus,
				int row, int column ) 
		{
			Vector vec = (Vector )model1.getDataVector().get( row );
			Facility fac = (Facility )vec.get( 1 );
			Color col = coldef.getFacilityColor( fac );
			setBackground( col );
			return this;
		}
	}

	class PriorityColorRenderer extends DefaultTableCellRenderer
	{
		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent( JTable table,
				Object value, boolean isSelected, boolean hasFocus,
				int row, int column ) 
		{
			Vector vec = (Vector )model2.getDataVector().get( row );
			Priority pri = (Priority )vec.get( 1 );
			Color col = coldef.getPriorityColor( pri );
			setBackground( col );
			return this;
		}
	}

	public GeneralPanel( Dialog owner, SyslogParam param,
							Vector<SyslogEvent> vecLog, SyslogTable table )
	{
		this.owner = owner;
		this.param = param;
		this.vecLog = vecLog;
		this.table = table;
		createControls();
	}

	public GeneralPanel( Frame owner, SyslogParam param,
							Vector<SyslogEvent> vecLog, SyslogTable table )
	{
		this.owner = owner;
		this.param = param;
		this.vecLog = vecLog;
		this.table = table;
		createControls();
	}

	public GeneralPanel( SyslogParam param,
							Vector<SyslogEvent> vecLog, SyslogTable table )
	{
		this.param = param;
		this.vecLog = vecLog;
		this.table = table;
		createControls();
	}

	protected void createControls() {
		res = ResourceBundle.getBundle( "resource.viewer" );
		String text;
		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );

		text = res.getString( "statistics.lbl1.text" );
		Box box1 = new Box( BoxLayout.X_AXIS );
		LBL_ALL = new JLabel( text );
		box1.add( LBL_ALL );
		TXT_ALL.setEditable( false );
		box1.add( TXT_ALL );
		box1.add( Box.createHorizontalStrut( 10 ) );

		text = res.getString( "statistics.lbl2.text" );
		LBL_DISP = new JLabel( text );
		box1.add( LBL_DISP );
		TXT_DISP.setEditable( false );
		box1.add( TXT_DISP );
/*
		box1.add( Box.createHorizontalStrut( 10 ) );

		BTN_UPD.setMnemonic( KeyEvent.VK_ESCAPE );
		BTN_UPD.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				received();
			}
		});
		box1.add( BTN_UPD );
*/
		add( box1 );

		JScrollPane sp;
		TBL_FAC.setRowSelectionAllowed( false );
		TBL_FAC.setCellSelectionEnabled( true );
		model1.addColumn( " " );
		model1.addColumn( "Facility" );
		model1.addColumn( "Count" );
		model1.addColumn( "%" );
		TableColumn col;
		col = TBL_FAC.getColumnModel().getColumn( 0 );
		col.setCellRenderer( new FacilityColorRenderer() );
		col.setMaxWidth( 20 );
		col = TBL_FAC.getColumnModel().getColumn( 2 );
		col.setCellRenderer( new NumberRenderer() );
		col.setMaxWidth( 100 );
		col = TBL_FAC.getColumnModel().getColumn( 3 );
		col.setCellRenderer( new NumberRenderer() );
		col.setMaxWidth( 50 );
		sp = new JScrollPane( TBL_FAC );
		sp.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS );
		sp.setBorder( new TitledBorder( "Facility" ));
		add( sp );

		TBL_PRI.setRowSelectionAllowed( false );
		TBL_PRI.setCellSelectionEnabled( true );
		model2.addColumn( " " );
		model2.addColumn( "Priority" );
		model2.addColumn( "Count" );
		model2.addColumn( "%" );
		col = TBL_PRI.getColumnModel().getColumn( 0 );
		col.setCellRenderer( new PriorityColorRenderer() );
		col.setMaxWidth( 20 );
		col = TBL_PRI.getColumnModel().getColumn( 2 );
		col.setCellRenderer( new NumberRenderer() );
		col.setMaxWidth( 100 );
		col = TBL_PRI.getColumnModel().getColumn( 3 );
		col.setCellRenderer( new NumberRenderer() );
		col.setMaxWidth( 50 );
		sp = new JScrollPane( TBL_PRI );
		sp.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS );
		sp.setBorder( new TitledBorder( "Priority" ));
		add( sp );
	}

	protected void received() {
		TXT_ALL.setText( String.valueOf( vecLog.size()));
		TXT_DISP.setText( String.valueOf( table.getModel().getRowCount()));
		model1.setRowCount( 0 );
		model2.setRowCount( 0 );
		mapFac.clear();
		mapPri.clear();
		for ( int i = 0; i < vecLog.size(); i++ ) {
			SyslogMsg msg = ((SyslogEvent )vecLog.get( i )).getMessage();
			Facility fac = msg.getFacility();
			Integer cnt;
			cnt = (Integer )mapFac.get( fac );
			if ( cnt == null ) {
				cnt = Integer.valueOf( 0 );
			}
			mapFac.put( fac, Integer.valueOf( cnt.intValue() + 1 ));

			Priority pri = msg.getPriority();
			cnt = (Integer )mapPri.get( pri );
			if ( cnt == null ) {
				cnt = Integer.valueOf( 0 );
			}
			mapPri.put( pri, Integer.valueOf( cnt.intValue() + 1 ));
		}

		Iterator<Facility> itrFac = mapFac.keySet().iterator();
		int nAll = vecLog.size();
		while ( itrFac.hasNext()) {
			Facility fac = (Facility )itrFac.next();
			int nNum = ((Integer )mapFac.get( fac )).intValue();
			double perc = (double )nNum / (double )nAll;
			model1.add( fac, Integer.valueOf( nNum ), new Double( perc ));
		}
		Iterator<Priority> itrPri = mapPri.keySet().iterator();
		while ( itrPri.hasNext()) {
			Priority pri = (Priority )itrPri.next();
			int nNum = ((Integer )mapPri.get( pri )).intValue();
			double perc = (double )nNum / (double )nAll;
			model2.add( pri, Integer.valueOf( nNum ), new Double( perc ));
		}
		TBL_FAC.revalidate();
		TBL_PRI.revalidate();
		repaint();
	}

}
